@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2009 Junjiro R. Okajima
+ * Copyright (C) 2005-2011 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,9 @@
  * branch management
  */
 
+#include <linux/compat.h>
 #include <linux/file.h>
+#include <linux/statfs.h>
 #include "aufs.h"
 
 /*
@@ -30,6 +32,9 @@ static void au_br_do_free(struct au_branch *br)
 {
 	int i;
 	struct au_wbr *wbr;
+	struct au_dykey **key;
+
+	au_hnotify_fin_br(br);
 
 	if (br->br_xino.xi_file)
 		fput(br->br_xino.xi_file);
@@ -42,14 +47,17 @@ static void au_br_do_free(struct au_branch *br)
 		for (i = 0; i < AuBrWh_Last; i++)
 			dput(wbr->wbr_wh[i]);
 		AuDebugOn(atomic_read(&wbr->wbr_wh_running));
-		au_rwsem_destroy(&wbr->wbr_wh_rwsem);
+		AuRwDestroy(&wbr->wbr_wh_rwsem);
 	}
 
-	/* some filesystems acquire extra lock */
-	lockdep_off();
-	mntput(br->br_mnt);
-	lockdep_on();
+	key = br->br_dykey;
+	for (i = 0; i < AuBrDynOp; i++, key++)
+		if (*key)
+			au_dy_put(*key);
+		else
+			break;
 
+	mntput(br->br_mnt);
 	kfree(wbr);
 	kfree(br);
 }
@@ -62,6 +70,8 @@ void au_br_free(struct au_sbinfo *sbinfo)
 	aufs_bindex_t bmax;
 	struct au_branch **br;
 
+	AuRwMustWriteLock(&sbinfo->si_rwsem);
+
 	bmax = sbinfo->si_bend + 1;
 	br = sbinfo->si_branch;
 	while (bmax--)
@@ -88,15 +98,16 @@ int au_br_index(struct super_block *sb, aufs_bindex_t br_id)
  * add a branch
  */
 
-static int test_overlap(struct super_block *sb, struct dentry *h_d1,
-			struct dentry *h_d2)
+static int test_overlap(struct super_block *sb, struct dentry *h_adding,
+			struct dentry *h_root)
 {
-	if (unlikely(h_d1 == h_d2))
+	if (unlikely(h_adding == h_root
+		     || au_test_loopback_overlap(sb, h_adding)))
 		return 1;
-	return !!au_test_subdir(h_d1, h_d2)
-		|| !!au_test_subdir(h_d2, h_d1)
-		|| au_test_loopback_overlap(sb, h_d1, h_d2)
-		|| au_test_loopback_overlap(sb, h_d2, h_d1);
+	if (h_adding->d_sb != h_root->d_sb)
+		return 0;
+	return au_test_subdir(h_adding, h_root)
+		|| au_test_subdir(h_root, h_adding);
 }
 
 /*
@@ -108,33 +119,43 @@ static struct au_branch *au_br_alloc(struct super_block *sb, int new_nbranch,
 {
 	struct au_branch *add_branch;
 	struct dentry *root;
+	int err;
 
+	err = -ENOMEM;
 	root = sb->s_root;
 	add_branch = kmalloc(sizeof(*add_branch), GFP_NOFS);
 	if (unlikely(!add_branch))
 		goto out;
 
+	err = au_hnotify_init_br(add_branch, perm);
+	if (unlikely(err))
+		goto out_br;
+
 	add_branch->br_wbr = NULL;
 	if (au_br_writable(perm)) {
 		/* may be freed separately at changing the branch permission */
 		add_branch->br_wbr = kmalloc(sizeof(*add_branch->br_wbr),
 					     GFP_NOFS);
 		if (unlikely(!add_branch->br_wbr))
-			goto out_br;
+			goto out_hnotify;
 	}
 
-	if (unlikely(au_sbr_realloc(au_sbi(sb), new_nbranch)
-		     || au_di_realloc(au_di(root), new_nbranch)
-		     || au_ii_realloc(au_ii(root->d_inode), new_nbranch)))
-		goto out_wbr;
-	return add_branch; /* success */
+	err = au_sbr_realloc(au_sbi(sb), new_nbranch);
+	if (!err)
+		err = au_di_realloc(au_di(root), new_nbranch);
+	if (!err)
+		err = au_ii_realloc(au_ii(root->d_inode), new_nbranch);
+	if (!err)
+		return add_branch; /* success */
 
- out_wbr:
 	kfree(add_branch->br_wbr);
- out_br:
+
+out_hnotify:
+	au_hnotify_fin_br(add_branch);
+out_br:
 	kfree(add_branch);
- out:
-	return ERR_PTR(-ENOMEM);
+out:
+	return ERR_PTR(err);
 }
 
 /*
@@ -144,13 +165,14 @@ static int test_br(struct inode *inode, int brperm, char *path)
 {
 	int err;
 
-	err = 0;
-	if (unlikely(au_br_writable(brperm) && IS_RDONLY(inode))) {
-		AuErr("write permission for readonly mount or inode, %s\n",
-		      path);
-		err = -EINVAL;
-	}
+	err = (au_br_writable(brperm) && IS_RDONLY(inode));
+	if (!err)
+		goto out;
 
+	err = -EINVAL;
+	pr_err("write permission for readonly mount or inode, %s\n", path);
+
+out:
 	return err;
 }
 
@@ -174,7 +196,7 @@ static int test_add(struct super_block *sb, struct au_opt_add *add, int remount)
 		err = 1;
 		if (!remount) {
 			err = -EINVAL;
-			AuErr("%s duplicated\n", add->pathname);
+			pr_err("%s duplicated\n", add->pathname);
 		}
 		goto out;
 	}
@@ -182,32 +204,32 @@ static int test_add(struct super_block *sb, struct au_opt_add *add, int remount)
 	err = -ENOSPC; /* -E2BIG; */
 	if (unlikely(AUFS_BRANCH_MAX <= add->bindex
 		     || AUFS_BRANCH_MAX - 1 <= bend)) {
-		AuErr("number of branches exceeded %s\n", add->pathname);
+		pr_err("number of branches exceeded %s\n", add->pathname);
 		goto out;
 	}
 
 	err = -EDOM;
 	if (unlikely(add->bindex < 0 || bend + 1 < add->bindex)) {
-		AuErr("bad index %d\n", add->bindex);
+		pr_err("bad index %d\n", add->bindex);
 		goto out;
 	}
 
 	inode = add->path.dentry->d_inode;
 	err = -ENOENT;
 	if (unlikely(!inode->i_nlink)) {
-		AuErr("no existence %s\n", add->pathname);
+		pr_err("no existence %s\n", add->pathname);
 		goto out;
 	}
 
 	err = -EINVAL;
 	if (unlikely(inode->i_sb == sb)) {
-		AuErr("%s must be outside\n", add->pathname);
+		pr_err("%s must be outside\n", add->pathname);
 		goto out;
 	}
 
 	if (unlikely(au_test_fs_unsuppoted(inode->i_sb))) {
-		AuErr("unsupported filesystem, %s (%s)\n",
-		      add->pathname, au_sbtype(inode->i_sb));
+		pr_err("unsupported filesystem, %s (%s)\n",
+		       add->pathname, au_sbtype(inode->i_sb));
 		goto out;
 	}
 
@@ -222,7 +244,7 @@ static int test_add(struct super_block *sb, struct au_opt_add *add, int remount)
 	for (bindex = 0; bindex <= bend; bindex++)
 		if (unlikely(test_overlap(sb, add->path.dentry,
 					  au_h_dptr(root, bindex)))) {
-			AuErr("%s is overlapped\n", add->pathname);
+			pr_err("%s is overlapped\n", add->pathname);
 			goto out;
 		}
 
@@ -232,15 +254,15 @@ static int test_add(struct super_block *sb, struct au_opt_add *add, int remount)
 		if ((h_inode->i_mode & S_IALLUGO) != (inode->i_mode & S_IALLUGO)
 		    || h_inode->i_uid != inode->i_uid
 		    || h_inode->i_gid != inode->i_gid)
-			AuWarn("uid/gid/perm %s %u/%u/0%o, %u/%u/0%o\n",
-			       add->pathname,
-			       inode->i_uid, inode->i_gid,
-			       (inode->i_mode & S_IALLUGO),
-			       h_inode->i_uid, h_inode->i_gid,
-			       (h_inode->i_mode & S_IALLUGO));
+			pr_warning("uid/gid/perm %s %u/%u/0%o, %u/%u/0%o\n",
+				   add->pathname,
+				   inode->i_uid, inode->i_gid,
+				   (inode->i_mode & S_IALLUGO),
+				   h_inode->i_uid, h_inode->i_gid,
+				   (h_inode->i_mode & S_IALLUGO));
 	}
 
- out:
+out:
 	return err;
 }
 
@@ -264,7 +286,7 @@ static int au_br_init_wh(struct super_block *sb, struct au_branch *br,
 	bindex = au_br_index(sb, br->br_id);
 	if (0 <= bindex) {
 		hdir = au_hi(sb->s_root->d_inode, bindex);
-		au_hin_imtx_lock_nested(hdir, AuLsc_I_PARENT);
+		au_hn_imtx_lock_nested(hdir, AuLsc_I_PARENT);
 	} else {
 		h_mtx = &h_root->d_inode->i_mutex;
 		mutex_lock_nested(h_mtx, AuLsc_I_PARENT);
@@ -277,7 +299,7 @@ static int au_br_init_wh(struct super_block *sb, struct au_branch *br,
 		wbr_wh_write_unlock(wbr);
 	}
 	if (hdir)
-		au_hin_imtx_unlock(hdir);
+		au_hn_imtx_unlock(hdir);
 	else
 		mutex_unlock(h_mtx);
 	br->br_perm = old_perm;
@@ -294,16 +316,33 @@ static int au_wbr_init(struct au_branch *br, struct super_block *sb,
 		       int perm, struct path *path)
 {
 	int err;
+	struct kstatfs kst;
 	struct au_wbr *wbr;
+	struct dentry *h_dentry;
 
 	wbr = br->br_wbr;
-	init_rwsem(&wbr->wbr_wh_rwsem);
+	au_rw_init(&wbr->wbr_wh_rwsem);
 	memset(wbr->wbr_wh, 0, sizeof(wbr->wbr_wh));
 	atomic_set(&wbr->wbr_wh_running, 0);
 	wbr->wbr_bytes = 0;
 
-	err = au_br_init_wh(sb, br, perm, path->dentry);
+	/*
+	 * a limit for rmdir/rename a dir
+	 * cf. AUFS_MAX_NAMELEN in include/linux/aufs_type.h
+	 */
+	err = vfs_statfs(path, &kst);
+	if (unlikely(err))
+		goto out;
+	err = -EINVAL;
+	h_dentry = path->dentry;
+	if (kst.f_namelen >= NAME_MAX)
+		err = au_br_init_wh(sb, br, perm, h_dentry);
+	else
+		pr_err("%.*s(%s), unsupported namelen %ld\n",
+		       AuDLNPair(h_dentry), au_sbtype(h_dentry->d_sb),
+		       kst.f_namelen);
 
+out:
 	return err;
 }
 
@@ -318,15 +357,18 @@ static int au_br_init(struct au_branch *br, struct super_block *sb,
 	mutex_init(&br->br_xino.xi_nondir_mtx);
 	br->br_perm = add->perm;
 	br->br_mnt = add->path.mnt; /* set first, mntget() later */
+	spin_lock_init(&br->br_dykey_lock);
+	memset(br->br_dykey, 0, sizeof(br->br_dykey));
 	atomic_set(&br->br_count, 0);
 	br->br_xino_upper = AUFS_XINO_TRUNC_INIT;
 	atomic_set(&br->br_xino_running, 0);
 	br->br_id = au_new_br_id(sb);
+	AuDebugOn(br->br_id < 0);
 
 	if (au_br_writable(add->perm)) {
 		err = au_wbr_init(br, sb, add->perm, &add->path);
 		if (unlikely(err))
-			goto out;
+			goto out_err;
 	}
 
 	if (au_opt_test(au_mntflags(sb), XINO)) {
@@ -334,14 +376,17 @@ static int au_br_init(struct au_branch *br, struct super_block *sb,
 				 au_sbr(sb, 0)->br_xino.xi_file, /*do_test*/1);
 		if (unlikely(err)) {
 			AuDebugOn(br->br_xino.xi_file);
-			goto out;
+			goto out_err;
 		}
 	}
 
 	sysaufs_br_init(br);
 	mntget(add->path.mnt);
+	goto out; /* success */
 
- out:
+out_err:
+	br->br_mnt = NULL;
+out:
 	return err;
 }
 
@@ -351,6 +396,8 @@ static void au_br_do_add_brp(struct au_sbinfo *sbinfo, aufs_bindex_t bindex,
 {
 	struct au_branch **brp;
 
+	AuRwMustWriteLock(&sbinfo->si_rwsem);
+
 	brp = sbinfo->si_branch + bindex;
 	memmove(brp + 1, brp, sizeof(*brp) * amount);
 	*brp = br;
@@ -364,6 +411,8 @@ static void au_br_do_add_hdp(struct au_dinfo *dinfo, aufs_bindex_t bindex,
 {
 	struct au_hdentry *hdp;
 
+	AuRwMustWriteLock(&dinfo->di_rwsem);
+
 	hdp = dinfo->di_hdentry + bindex;
 	memmove(hdp + 1, hdp, sizeof(*hdp) * amount);
 	au_h_dentry_init(hdp);
@@ -377,10 +426,12 @@ static void au_br_do_add_hip(struct au_iinfo *iinfo, aufs_bindex_t bindex,
 {
 	struct au_hinode *hip;
 
+	AuRwMustWriteLock(&iinfo->ii_rwsem);
+
 	hip = iinfo->ii_hinode + bindex;
 	memmove(hip + 1, hip, sizeof(*hip) * amount);
 	hip->hi_inode = NULL;
-	au_hin_init(hip, NULL);
+	au_hn_init(hip);
 	iinfo->ii_bend++;
 	if (unlikely(bend < 0))
 		iinfo->ii_bstart = 0;
@@ -395,21 +446,21 @@ static void au_br_do_add(struct super_block *sb, struct dentry *h_dentry,
 
 	root = sb->s_root;
 	root_inode = root->d_inode;
-	au_plink_block_maintain(sb);
 	bend = au_sbend(sb);
 	amount = bend + 1 - bindex;
+	au_sbilist_lock();
 	au_br_do_add_brp(au_sbi(sb), bindex, br, bend, amount);
 	au_br_do_add_hdp(au_di(root), bindex, bend, amount);
 	au_br_do_add_hip(au_ii(root_inode), bindex, bend, amount);
 	au_set_h_dptr(root, bindex, dget(h_dentry));
 	au_set_h_iptr(root_inode, bindex, au_igrab(h_dentry->d_inode),
 		      /*flags*/0);
+	au_sbilist_unlock();
 }
 
 int au_br_add(struct super_block *sb, struct au_opt_add *add, int remount)
 {
 	int err;
-	unsigned long long maxb;
 	aufs_bindex_t bend, add_bindex;
 	struct dentry *root, *h_dentry;
 	struct inode *root_inode;
@@ -448,17 +499,15 @@ int au_br_add(struct super_block *sb, struct au_opt_add *add, int remount)
 		sysaufs_brs_add(sb, add_bindex);
 	}
 
-	if (!add_bindex)
+	if (!add_bindex) {
 		au_cpup_attr_all(root_inode, /*force*/1);
-	else
+		sb->s_maxbytes = h_dentry->d_sb->s_maxbytes;
+	} else
 		au_add_nlink(root_inode, h_dentry->d_inode);
-	maxb = h_dentry->d_sb->s_maxbytes;
-	if (sb->s_maxbytes < maxb)
-		sb->s_maxbytes = maxb;
 
 	/*
-	 * this test/set prevents aufs from handling unnecesary inotify events
-	 * of xino files, in a case of re-adding a writable branch which was
+	 * this test/set prevents aufs from handling unnecesary notify events
+	 * of xino files, in case of re-adding a writable branch which was
 	 * once detached from aufs.
 	 */
 	if (au_xino_brid(sb) < 0
@@ -468,7 +517,7 @@ int au_br_add(struct super_block *sb, struct au_opt_add *add, int remount)
 	    && add_branch->br_xino.xi_file->f_dentry->d_parent == h_dentry)
 		au_xino_brid_set(sb, add_branch->br_id);
 
- out:
+out:
 	return err;
 }
 
@@ -479,24 +528,34 @@ int au_br_add(struct super_block *sb, struct au_opt_add *add, int remount)
  */
 
 /* to show the line number, do not make it inlined function */
-#define AuVerbose(do_info, fmt, args...) do { \
+#define AuVerbose(do_info, fmt, ...) do { \
 	if (do_info) \
-		AuInfo(fmt, ##args); \
+		pr_info(fmt, ##__VA_ARGS__); \
 } while (0)
 
+static int au_test_ibusy(struct inode *inode, aufs_bindex_t bstart,
+			 aufs_bindex_t bend)
+{
+	return (inode && !S_ISDIR(inode->i_mode)) || bstart == bend;
+}
+
+static int au_test_dbusy(struct dentry *dentry, aufs_bindex_t bstart,
+			 aufs_bindex_t bend)
+{
+	return au_test_ibusy(dentry->d_inode, bstart, bend);
+}
+
 /*
  * test if the branch is deletable or not.
  */
 static int test_dentry_busy(struct dentry *root, aufs_bindex_t bindex,
-			    unsigned int sigen)
+			    unsigned int sigen, const unsigned int verbose)
 {
 	int err, i, j, ndentry;
 	aufs_bindex_t bstart, bend;
-	unsigned char verbose;
 	struct au_dcsub_pages dpages;
 	struct au_dpage *dpage;
 	struct dentry *d;
-	struct inode *inode;
 
 	err = au_dpages_init(&dpages, GFP_NOFS);
 	if (unlikely(err))
@@ -505,18 +564,24 @@ static int test_dentry_busy(struct dentry *root, aufs_bindex_t bindex,
 	if (unlikely(err))
 		goto out_dpages;
 
-	verbose = !!au_opt_test(au_mntflags(root->d_sb), VERBOSE);
 	for (i = 0; !err && i < dpages.ndpage; i++) {
 		dpage = dpages.dpages + i;
 		ndentry = dpage->ndentry;
 		for (j = 0; !err && j < ndentry; j++) {
 			d = dpage->dentries[j];
-			AuDebugOn(!atomic_read(&d->d_count));
-			inode = d->d_inode;
-			if (au_digen(d) == sigen && au_iigen(inode) == sigen)
+			AuDebugOn(!d->d_count);
+			if (!au_digen_test(d, sigen)) {
 				di_read_lock_child(d, AuLock_IR);
-			else {
+				if (unlikely(au_dbrange_test(d))) {
+					di_read_unlock(d, AuLock_IR);
+					continue;
+				}
+			} else {
 				di_write_lock_child(d);
+				if (unlikely(au_dbrange_test(d))) {
+					di_write_unlock(d);
+					continue;
+				}
 				err = au_reval_dpath(d, sigen);
 				if (!err)
 					di_downgrade_lock(d, AuLock_IR);
@@ -526,45 +591,54 @@ static int test_dentry_busy(struct dentry *root, aufs_bindex_t bindex,
 				}
 			}
 
+			/* AuDbgDentry(d); */
 			bstart = au_dbstart(d);
 			bend = au_dbend(d);
 			if (bstart <= bindex
 			    && bindex <= bend
 			    && au_h_dptr(d, bindex)
-			    && (!S_ISDIR(inode->i_mode) || bstart == bend)) {
+			    && au_test_dbusy(d, bstart, bend)) {
 				err = -EBUSY;
 				AuVerbose(verbose, "busy %.*s\n", AuDLNPair(d));
+				AuDbgDentry(d);
 			}
 			di_read_unlock(d, AuLock_IR);
 		}
 	}
 
- out_dpages:
+out_dpages:
 	au_dpages_free(&dpages);
- out:
+out:
 	return err;
 }
 
 static int test_inode_busy(struct super_block *sb, aufs_bindex_t bindex,
-			   unsigned int sigen)
+			   unsigned int sigen, const unsigned int verbose)
 {
 	int err;
-	struct inode *i;
+	unsigned long long max, ull;
+	struct inode *i, **array;
 	aufs_bindex_t bstart, bend;
-	unsigned char verbose;
+
+	array = au_iarray_alloc(sb, &max);
+	err = PTR_ERR(array);
+	if (IS_ERR(array))
+		goto out;
 
 	err = 0;
-	verbose = !!au_opt_test(au_mntflags(sb), VERBOSE);
-	list_for_each_entry(i, &sb->s_inodes, i_sb_list) {
-		AuDebugOn(!atomic_read(&i->i_count));
-		if (!list_empty(&i->i_dentry))
+	AuDbg("b%d\n", bindex);
+	for (ull = 0; !err && ull < max; ull++) {
+		i = array[ull];
+		if (i->i_ino == AUFS_ROOT_INO)
 			continue;
 
+		/* AuDbgInode(i); */
 		if (au_iigen(i) == sigen)
 			ii_read_lock_child(i);
 		else {
 			ii_write_lock_child(i);
-			err = au_refresh_hinode_self(i, /*do_attr*/1);
+			err = au_refresh_hinode_self(i);
+			au_iigen_dec(i);
 			if (!err)
 				ii_downgrade_lock(i);
 			else {
@@ -578,19 +652,21 @@ static int test_inode_busy(struct super_block *sb, aufs_bindex_t bindex,
 		if (bstart <= bindex
 		    && bindex <= bend
 		    && au_h_iptr(i, bindex)
-		    && (!S_ISDIR(i->i_mode) || bstart == bend)) {
+		    && au_test_ibusy(i, bstart, bend)) {
 			err = -EBUSY;
 			AuVerbose(verbose, "busy i%lu\n", i->i_ino);
-			ii_read_unlock(i);
-			break;
+			AuDbgInode(i);
 		}
 		ii_read_unlock(i);
 	}
+	au_iarray_free(array, max);
 
+out:
 	return err;
 }
 
-static int test_children_busy(struct dentry *root, aufs_bindex_t bindex)
+static int test_children_busy(struct dentry *root, aufs_bindex_t bindex,
+			      const unsigned int verbose)
 {
 	int err;
 	unsigned int sigen;
@@ -599,9 +675,9 @@ static int test_children_busy(struct dentry *root, aufs_bindex_t bindex)
 	DiMustNoWaiters(root);
 	IiMustNoWaiters(root->d_inode);
 	di_write_unlock(root);
-	err = test_dentry_busy(root, bindex, sigen);
+	err = test_dentry_busy(root, bindex, sigen, verbose);
 	if (!err)
-		err = test_inode_busy(root->d_sb, bindex, sigen);
+		err = test_inode_busy(root->d_sb, bindex, sigen, verbose);
 	di_write_lock_child(root); /* aufs_write_lock() calls ..._child() */
 
 	return err;
@@ -613,15 +689,18 @@ static void au_br_do_del_brp(struct au_sbinfo *sbinfo,
 {
 	struct au_branch **brp, **p;
 
+	AuRwMustWriteLock(&sbinfo->si_rwsem);
+
 	brp = sbinfo->si_branch + bindex;
 	if (bindex < bend)
 		memmove(brp, brp + 1, sizeof(*brp) * (bend - bindex));
 	sbinfo->si_branch[0 + bend] = NULL;
 	sbinfo->si_bend--;
 
-	p = krealloc(sbinfo->si_branch, sizeof(*p) * bend, GFP_NOFS);
+	p = krealloc(sbinfo->si_branch, sizeof(*p) * bend, AuGFP_SBILIST);
 	if (p)
 		sbinfo->si_branch = p;
+	/* harmless error */
 }
 
 static void au_br_do_del_hdp(struct au_dinfo *dinfo, const aufs_bindex_t bindex,
@@ -629,15 +708,19 @@ static void au_br_do_del_hdp(struct au_dinfo *dinfo, const aufs_bindex_t bindex,
 {
 	struct au_hdentry *hdp, *p;
 
-	hdp = dinfo->di_hdentry + bindex;
+	AuRwMustWriteLock(&dinfo->di_rwsem);
+
+	hdp = dinfo->di_hdentry;
 	if (bindex < bend)
-		memmove(hdp, hdp + 1, sizeof(*hdp) * (bend - bindex));
-	dinfo->di_hdentry[0 + bend].hd_dentry = NULL;
+		memmove(hdp + bindex, hdp + bindex + 1,
+			sizeof(*hdp) * (bend - bindex));
+	hdp[0 + bend].hd_dentry = NULL;
 	dinfo->di_bend--;
 
-	p = krealloc(dinfo->di_hdentry, sizeof(*p) * bend, GFP_NOFS);
+	p = krealloc(hdp, sizeof(*p) * bend, AuGFP_SBILIST);
 	if (p)
 		dinfo->di_hdentry = p;
+	/* harmless error */
 }
 
 static void au_br_do_del_hip(struct au_iinfo *iinfo, const aufs_bindex_t bindex,
@@ -645,16 +728,19 @@ static void au_br_do_del_hip(struct au_iinfo *iinfo, const aufs_bindex_t bindex,
 {
 	struct au_hinode *hip, *p;
 
+	AuRwMustWriteLock(&iinfo->ii_rwsem);
+
 	hip = iinfo->ii_hinode + bindex;
 	if (bindex < bend)
 		memmove(hip, hip + 1, sizeof(*hip) * (bend - bindex));
 	iinfo->ii_hinode[0 + bend].hi_inode = NULL;
-	au_hin_init(iinfo->ii_hinode + bend, NULL);
+	au_hn_init(iinfo->ii_hinode + bend);
 	iinfo->ii_bend--;
 
-	p = krealloc(iinfo->ii_hinode, sizeof(*p) * bend, GFP_NOFS);
+	p = krealloc(iinfo->ii_hinode, sizeof(*p) * bend, AuGFP_SBILIST);
 	if (p)
 		iinfo->ii_hinode = p;
+	/* harmless error */
 }
 
 static void au_br_do_del(struct super_block *sb, aufs_bindex_t bindex,
@@ -662,22 +748,31 @@ static void au_br_do_del(struct super_block *sb, aufs_bindex_t bindex,
 {
 	aufs_bindex_t bend;
 	struct au_sbinfo *sbinfo;
-	struct dentry *root;
-	struct inode *inode;
+	struct dentry *root, *h_root;
+	struct inode *inode, *h_inode;
+	struct au_hinode *hinode;
+
+	SiMustWriteLock(sb);
 
 	root = sb->s_root;
 	inode = root->d_inode;
-	au_plink_block_maintain(sb);
 	sbinfo = au_sbi(sb);
 	bend = sbinfo->si_bend;
 
-	dput(au_h_dptr(root, bindex));
-	au_hiput(au_hi(inode, bindex));
-	au_br_do_free(br);
+	h_root = au_h_dptr(root, bindex);
+	hinode = au_hi(inode, bindex);
+	h_inode = au_igrab(hinode->hi_inode);
+	au_hiput(hinode);
 
+	au_sbilist_lock();
 	au_br_do_del_brp(sbinfo, bindex, bend);
 	au_br_do_del_hdp(au_di(root), bindex, bend);
 	au_br_do_del_hip(au_ii(inode), bindex, bend);
+	au_sbilist_unlock();
+
+	dput(h_root);
+	iput(h_inode);
+	au_br_do_free(br);
 }
 
 int au_br_del(struct super_block *sb, struct au_opt_del *del, int remount)
@@ -695,7 +790,7 @@ int au_br_del(struct super_block *sb, struct au_opt_del *del, int remount)
 		if (remount)
 			goto out; /* success */
 		err = -ENOENT;
-		AuErr("%s no such branch\n", del->pathname);
+		pr_err("%s no such branch\n", del->pathname);
 		goto out;
 	}
 	AuDbg("bindex b%d\n", bindex);
@@ -718,13 +813,15 @@ int au_br_del(struct super_block *sb, struct au_opt_del *del, int remount)
 	wbr = br->br_wbr;
 	do_wh = wbr && (wbr->wbr_whbase || wbr->wbr_plink || wbr->wbr_orph);
 	if (do_wh) {
+		/* instead of WbrWhMustWriteLock(wbr) */
+		SiMustWriteLock(sb);
 		for (i = 0; i < AuBrWh_Last; i++) {
 			dput(wbr->wbr_wh[i]);
 			wbr->wbr_wh[i] = NULL;
 		}
 	}
 
-	err = test_children_busy(sb->s_root, bindex);
+	err = test_children_busy(sb->s_root, bindex, verbose);
 	if (unlikely(err)) {
 		if (do_wh)
 			goto out_wh;
@@ -741,45 +838,113 @@ int au_br_del(struct super_block *sb, struct au_opt_del *del, int remount)
 		sysaufs_brs_add(sb, bindex);
 	}
 
-	if (!bindex)
+	if (!bindex) {
 		au_cpup_attr_all(sb->s_root->d_inode, /*force*/1);
-	else
+		sb->s_maxbytes = au_sbr_sb(sb, 0)->s_maxbytes;
+	} else
 		au_sub_nlink(sb->s_root->d_inode, del->h_path.dentry->d_inode);
 	if (au_opt_test(mnt_flags, PLINK))
 		au_plink_half_refresh(sb, br_id);
 
-	if (sb->s_maxbytes == del->h_path.dentry->d_sb->s_maxbytes) {
-		bend--;
-		sb->s_maxbytes = 0;
-		for (bindex = 0; bindex <= bend; bindex++) {
-			unsigned long long maxb;
-
-			maxb = au_sbr_sb(sb, bindex)->s_maxbytes;
-			if (sb->s_maxbytes < maxb)
-				sb->s_maxbytes = maxb;
-		}
-	}
-
-	if (au_xino_brid(sb) == br->br_id)
+	if (au_xino_brid(sb) == br_id)
 		au_xino_brid_set(sb, -1);
 	goto out; /* success */
 
- out_wh:
+out_wh:
 	/* revert */
 	rerr = au_br_init_wh(sb, br, br->br_perm, del->h_path.dentry);
 	if (rerr)
-		AuWarn("failed re-creating base whiteout, %s. (%d)\n",
-		       del->pathname, rerr);
- out:
+		pr_warning("failed re-creating base whiteout, %s. (%d)\n",
+			   del->pathname, rerr);
+out:
 	return err;
 }
 
 /* ---------------------------------------------------------------------- */
 
+static int au_ibusy(struct super_block *sb, struct aufs_ibusy __user *arg)
+{
+	int err;
+	aufs_bindex_t bstart, bend;
+	struct aufs_ibusy ibusy;
+	struct inode *inode, *h_inode;
+
+	err = -EPERM;
+	if (unlikely(!capable(CAP_SYS_ADMIN)))
+		goto out;
+
+	err = copy_from_user(&ibusy, arg, sizeof(ibusy));
+	if (!err)
+		err = !access_ok(VERIFY_WRITE, &arg->h_ino, sizeof(arg->h_ino));
+	if (unlikely(err)) {
+		err = -EFAULT;
+		AuTraceErr(err);
+		goto out;
+	}
+
+	err = -EINVAL;
+	si_read_lock(sb, AuLock_FLUSH);
+	if (unlikely(ibusy.bindex < 0 || ibusy.bindex > au_sbend(sb)))
+		goto out_unlock;
+
+	err = 0;
+	ibusy.h_ino = 0; /* invalid */
+	inode = ilookup(sb, ibusy.ino);
+	if (!inode
+	    || inode->i_ino == AUFS_ROOT_INO
+	    || is_bad_inode(inode))
+		goto out_unlock;
+
+	ii_read_lock_child(inode);
+	bstart = au_ibstart(inode);
+	bend = au_ibend(inode);
+	if (bstart <= ibusy.bindex && ibusy.bindex <= bend) {
+		h_inode = au_h_iptr(inode, ibusy.bindex);
+		if (h_inode && au_test_ibusy(inode, bstart, bend))
+			ibusy.h_ino = h_inode->i_ino;
+	}
+	ii_read_unlock(inode);
+	iput(inode);
+
+out_unlock:
+	si_read_unlock(sb);
+	if (!err) {
+		err = __put_user(ibusy.h_ino, &arg->h_ino);
+		if (unlikely(err)) {
+			err = -EFAULT;
+			AuTraceErr(err);
+		}
+	}
+out:
+	return err;
+}
+
+long au_ibusy_ioctl(struct file *file, unsigned long arg)
+{
+	return au_ibusy(file->f_dentry->d_sb, (void __user *)arg);
+}
+
+#ifdef CONFIG_COMPAT
+long au_ibusy_compat_ioctl(struct file *file, unsigned long arg)
+{
+	return au_ibusy(file->f_dentry->d_sb, compat_ptr(arg));
+}
+#endif
+
+/* ---------------------------------------------------------------------- */
+
 /*
  * change a branch permission
  */
 
+static void au_warn_ima(void)
+{
+#ifdef CONFIG_IMA
+	/* since it doesn't support mark_files_ro() */
+	AuWarn1("RW -> RO makes IMA to produce wrong message\n");
+#endif
+}
+
 static int do_need_sigen_inc(int a, int b)
 {
 	return au_br_whable(a) && !au_br_whable(b);
@@ -791,101 +956,150 @@ static int need_sigen_inc(int old, int new)
 		|| do_need_sigen_inc(new, old);
 }
 
+static unsigned long long au_farray_cb(void *a,
+				       unsigned long long max __maybe_unused,
+				       void *arg)
+{
+	unsigned long long n;
+	struct file **p, *f;
+	struct super_block *sb = arg;
+
+	n = 0;
+	p = a;
+	lg_global_lock(files_lglock);
+	do_file_list_for_each_entry(sb, f) {
+		if (au_fi(f)
+		    && file_count(f)
+		    && !special_file(f->f_dentry->d_inode->i_mode)) {
+			get_file(f);
+			*p++ = f;
+			n++;
+			AuDebugOn(n > max);
+		}
+	} while_file_list_for_each_entry;
+	lg_global_unlock(files_lglock);
+
+	return n;
+}
+
+static struct file **au_farray_alloc(struct super_block *sb,
+				     unsigned long long *max)
+{
+	*max = atomic_long_read(&au_sbi(sb)->si_nfiles);
+	return au_array_alloc(max, au_farray_cb, sb);
+}
+
+static void au_farray_free(struct file **a, unsigned long long max)
+{
+	unsigned long long ull;
+
+	for (ull = 0; ull < max; ull++)
+		if (a[ull])
+			fput(a[ull]);
+	au_array_free(a);
+}
+
 static int au_br_mod_files_ro(struct super_block *sb, aufs_bindex_t bindex)
 {
-	int err;
-	unsigned long n, ul, bytes, files;
-	aufs_bindex_t bstart;
-	struct file *file, *hf, **a;
-	const int step_bytes = 1024, /* memory allocation unit */
-		step_files = step_bytes / sizeof(*a);
+	int err, do_warn;
+	unsigned int mnt_flags;
+	unsigned long long ull, max;
+	aufs_bindex_t br_id;
+	unsigned char verbose;
+	struct file *file, *hf, **array;
+	struct inode *inode;
+	struct au_hfile *hfile;
 
-	err = -ENOMEM;
-	n = 0;
-	bytes = step_bytes;
-	files = step_files;
-	a = kmalloc(bytes, GFP_NOFS);
-	if (unlikely(!a))
+	mnt_flags = au_mntflags(sb);
+	verbose = !!au_opt_test(mnt_flags, VERBOSE);
+
+	array = au_farray_alloc(sb, &max);
+	err = PTR_ERR(array);
+	if (IS_ERR(array))
 		goto out;
 
-	/* no need file_list_lock() since sbinfo is locked? defered? */
-	list_for_each_entry(file, &sb->s_files, f_u.fu_list) {
-		if (special_file(file->f_dentry->d_inode->i_mode))
-			continue;
+	do_warn = 0;
+	br_id = au_sbr_id(sb, bindex);
+	for (ull = 0; ull < max; ull++) {
+		file = array[ull];
 
-		AuDbg("%.*s\n", AuDLNPair(file->f_dentry));
+		/* AuDbg("%.*s\n", AuDLNPair(file->f_dentry)); */
 		fi_read_lock(file);
 		if (unlikely(au_test_mmapped(file))) {
 			err = -EBUSY;
+			AuVerbose(verbose, "mmapped %.*s\n",
+				  AuDLNPair(file->f_dentry));
+			AuDbgFile(file);
 			FiMustNoWaiters(file);
 			fi_read_unlock(file);
-			goto out_free;
+			goto out_array;
 		}
 
-		bstart = au_fbstart(file);
-		if (!S_ISREG(file->f_dentry->d_inode->i_mode)
+		inode = file->f_dentry->d_inode;
+		hfile = &au_fi(file)->fi_htop;
+		hf = hfile->hf_file;
+		if (!S_ISREG(inode->i_mode)
 		    || !(file->f_mode & FMODE_WRITE)
-		    || bstart != bindex) {
-			FiMustNoWaiters(file);
-			fi_read_unlock(file);
-			continue;
+		    || hfile->hf_br->br_id != br_id
+		    || !(hf->f_mode & FMODE_WRITE))
+			array[ull] = NULL;
+		else {
+			do_warn = 1;
+			get_file(file);
 		}
 
-		hf = au_h_fptr(file, bstart);
 		FiMustNoWaiters(file);
 		fi_read_unlock(file);
-
-		if (n < files)
-			a[n++] = hf;
-		else {
-			void *p;
-
-			err = -ENOMEM;
-			bytes += step_bytes;
-			files += step_files;
-			p = krealloc(a, bytes, GFP_NOFS);
-			if (p) {
-				a = p;
-				a[n++] = hf;
-			} else
-				goto out_free;
-		}
+		fput(file);
 	}
 
 	err = 0;
-	for (ul = 0; ul < n; ul++) {
+	if (do_warn)
+		au_warn_ima();
+
+	for (ull = 0; ull < max; ull++) {
+		file = array[ull];
+		if (!file)
+			continue;
+
 		/* todo: already flushed? */
 		/* cf. fs/super.c:mark_files_ro() */
-		hf = a[ul];
+		/* fi_read_lock(file); */
+		hfile = &au_fi(file)->fi_htop;
+		hf = hfile->hf_file;
+		/* fi_read_unlock(file); */
+		spin_lock(&hf->f_lock);
 		hf->f_mode &= ~FMODE_WRITE;
+		spin_unlock(&hf->f_lock);
 		if (!file_check_writeable(hf)) {
 			file_release_write(hf);
 			mnt_drop_write(hf->f_vfsmnt);
 		}
 	}
 
- out_free:
-	kfree(a);
- out:
+out_array:
+	au_farray_free(array, max);
+out:
+	AuTraceErr(err);
 	return err;
 }
 
 int au_br_mod(struct super_block *sb, struct au_opt_mod *mod, int remount,
-	      int *do_update)
+	      int *do_refresh)
 {
 	int err, rerr;
 	aufs_bindex_t bindex;
+	struct path path;
 	struct dentry *root;
 	struct au_branch *br;
 
 	root = sb->s_root;
-	au_plink_block_maintain(sb);
 	bindex = au_find_dbindex(root, mod->h_root);
 	if (bindex < 0) {
 		if (remount)
 			return 0; /* success */
 		err = -ENOENT;
-		AuErr("%s no such branch\n", mod->path);
+		pr_err("%s no such branch\n", mod->path);
 		goto out;
 	}
 	AuDbg("bindex b%d\n", bindex);
@@ -917,10 +1131,12 @@ int au_br_mod(struct super_block *sb, struct au_opt_mod *mod, int remount,
 				rerr = -ENOMEM;
 				br->br_wbr = kmalloc(sizeof(*br->br_wbr),
 						     GFP_NOFS);
-				if (br->br_wbr)
-					rerr = au_br_init_wh
-						(sb, br, br->br_perm,
-						 mod->h_root);
+				if (br->br_wbr) {
+					path.mnt = br->br_mnt;
+					path.dentry = mod->h_root;
+					rerr = au_wbr_init(br, sb, br->br_perm,
+							   &path);
+				}
 				if (unlikely(rerr)) {
 					AuIOErr("nested error %d (%d)\n",
 						rerr, err);
@@ -933,11 +1149,8 @@ int au_br_mod(struct super_block *sb, struct au_opt_mod *mod, int remount,
 		err = -ENOMEM;
 		br->br_wbr = kmalloc(sizeof(*br->br_wbr), GFP_NOFS);
 		if (br->br_wbr) {
-			struct path path = {
-				.mnt	= br->br_mnt,
-				.dentry	= mod->h_root
-			};
-
+			path.mnt = br->br_mnt;
+			path.dentry = mod->h_root;
 			err = au_wbr_init(br, sb, mod->perm, &path);
 			if (unlikely(err)) {
 				kfree(br->br_wbr);
@@ -947,10 +1160,11 @@ int au_br_mod(struct super_block *sb, struct au_opt_mod *mod, int remount,
 	}
 
 	if (!err) {
-		*do_update |= need_sigen_inc(br->br_perm, mod->perm);
+		*do_refresh |= need_sigen_inc(br->br_perm, mod->perm);
 		br->br_perm = mod->perm;
 	}
 
- out:
+out:
+	AuTraceErr(err);
 	return err;
 }
