@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2005-2010 Junjiro R. Okajima
+ * Copyright (C) 2005-2011 Junjiro R. Okajima
  *
  * This program, aufs is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -44,7 +44,7 @@ int au_do_open_nondir(struct file *file, int flags)
 
 	finfo = au_fi(file);
 	memset(&finfo->fi_htop, 0, sizeof(finfo->fi_htop));
-	finfo->fi_hvmop = NULL;
+	atomic_set(&finfo->fi_mmapped, 0);
 	bindex = au_dbstart(dentry);
 	h_file = au_h_open(dentry, bindex, flags, file);
 	if (IS_ERR(h_file))
@@ -67,7 +67,7 @@ static int aufs_open_nondir(struct inode *inode __maybe_unused,
 	int err;
 	struct super_block *sb;
 
-	AuDbg("%.*s, f_ flags 0x%x, f_mode 0x%x\n",
+	AuDbg("%.*s, f_flags 0x%x, f_mode 0x%x\n",
 	      AuDLNPair(file->f_dentry), vfsub_file_flags(file),
 	      file->f_mode);
 
@@ -115,6 +115,13 @@ static int aufs_flush_nondir(struct file *file, fl_owner_t id)
 }
 
 /* ---------------------------------------------------------------------- */
+/*
+ * read and write functions acquire [fdi]_rwsem once, but release before
+ * mmap_sem. This is because to stop a race condition between mmap(2).
+ * Releasing these aufs-rwsem should be safe, no branch-mamagement (by keeping
+ * si_rwsem), no harmful copy-up should happen. Actually copy-up may happen in
+ * read functions after [fdi]_rwsem are released, but it should be harmless.
+ */
 
 static ssize_t aufs_read(struct file *file, char __user *buf, size_t count,
 			 loff_t *ppos)
@@ -132,13 +139,18 @@ static ssize_t aufs_read(struct file *file, char __user *buf, size_t count,
 		goto out;
 
 	h_file = au_hf_top(file);
+	get_file(h_file);
+	di_read_unlock(dentry, AuLock_IR);
+	fi_read_unlock(file);
+
+	/* filedata may be obsoleted by concurrent copyup, but no problem */
 	err = vfsub_read_u(h_file, buf, count, ppos);
 	/* todo: necessary? */
 	/* file->f_ra = h_file->f_ra; */
+	/* update without lock, I don't think it a problem */
 	fsstack_copy_attr_atime(dentry->d_inode, h_file->f_dentry->d_inode);
+	fput(h_file);
 
-	di_read_unlock(dentry, AuLock_IR);
-	fi_read_unlock(file);
 out:
 	si_read_unlock(sb);
 	return err;
@@ -172,11 +184,13 @@ static ssize_t aufs_write(struct file *file, const char __user *ubuf,
 	ssize_t err;
 	struct au_pin pin;
 	struct dentry *dentry;
+	struct super_block *sb;
 	struct inode *inode;
 	struct file *h_file;
 	char __user *buf = (char __user *)ubuf;
 
 	dentry = file->f_dentry;
+	sb = dentry->d_sb;
 	inode = dentry->d_inode;
 	au_mtx_and_read_lock(inode);
 
@@ -186,20 +200,27 @@ static ssize_t aufs_write(struct file *file, const char __user *ubuf,
 
 	err = au_ready_to_write(file, -1, &pin);
 	di_downgrade_lock(dentry, AuLock_IR);
-	if (unlikely(err))
-		goto out_unlock;
+	if (unlikely(err)) {
+		di_read_unlock(dentry, AuLock_IR);
+		fi_write_unlock(file);
+		goto out;
+	}
 
 	h_file = au_hf_top(file);
+	get_file(h_file);
 	au_unpin(&pin);
-	err = vfsub_write_u(h_file, buf, count, ppos);
-	au_cpup_attr_timesizes(inode);
-	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
-
-out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
+
+	err = vfsub_write_u(h_file, buf, count, ppos);
+	ii_write_lock_child(inode);
+	au_cpup_attr_timesizes(inode);
+	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
+	ii_write_unlock(inode);
+	fput(h_file);
+
 out:
-	si_read_unlock(inode->i_sb);
+	si_read_unlock(sb);
 	mutex_unlock(&inode->i_mutex);
 	return err;
 }
@@ -225,7 +246,9 @@ static ssize_t au_do_aio(struct file *h_file, int rw, struct kiocb *kio,
 	if (func) {
 		file = kio->ki_filp;
 		kio->ki_filp = h_file;
+		lockdep_off();
 		err = func(kio, iov, nv, pos);
+		lockdep_on();
 		kio->ki_filp = file;
 	} else
 		/* currently there is no such fs */
@@ -252,12 +275,16 @@ static ssize_t aufs_aio_read(struct kiocb *kio, const struct iovec *iov,
 		goto out;
 
 	h_file = au_hf_top(file);
+	get_file(h_file);
+	di_read_unlock(dentry, AuLock_IR);
+	fi_read_unlock(file);
+
 	err = au_do_aio(h_file, MAY_READ, kio, iov, nv, pos);
 	/* todo: necessary? */
 	/* file->f_ra = h_file->f_ra; */
+	/* update without lock, I don't think it a problem */
 	fsstack_copy_attr_atime(dentry->d_inode, h_file->f_dentry->d_inode);
-	di_read_unlock(dentry, AuLock_IR);
-	fi_read_unlock(file);
+	fput(h_file);
 
 out:
 	si_read_unlock(sb);
@@ -272,9 +299,11 @@ static ssize_t aufs_aio_write(struct kiocb *kio, const struct iovec *iov,
 	struct dentry *dentry;
 	struct inode *inode;
 	struct file *file, *h_file;
+	struct super_block *sb;
 
 	file = kio->ki_filp;
 	dentry = file->f_dentry;
+	sb = dentry->d_sb;
 	inode = dentry->d_inode;
 	au_mtx_and_read_lock(inode);
 
@@ -284,20 +313,27 @@ static ssize_t aufs_aio_write(struct kiocb *kio, const struct iovec *iov,
 
 	err = au_ready_to_write(file, -1, &pin);
 	di_downgrade_lock(dentry, AuLock_IR);
-	if (unlikely(err))
-		goto out_unlock;
+	if (unlikely(err)) {
+		di_read_unlock(dentry, AuLock_IR);
+		fi_write_unlock(file);
+		goto out;
+	}
 
-	au_unpin(&pin);
 	h_file = au_hf_top(file);
-	err = au_do_aio(h_file, MAY_WRITE, kio, iov, nv, pos);
-	au_cpup_attr_timesizes(inode);
-	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
-
-out_unlock:
+	get_file(h_file);
+	au_unpin(&pin);
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
+
+	err = au_do_aio(h_file, MAY_WRITE, kio, iov, nv, pos);
+	ii_write_lock_child(inode);
+	au_cpup_attr_timesizes(inode);
+	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
+	ii_write_unlock(inode);
+	fput(h_file);
+
 out:
-	si_read_unlock(inode->i_sb);
+	si_read_unlock(sb);
 	mutex_unlock(&inode->i_mutex);
 	return err;
 }
@@ -320,17 +356,23 @@ static ssize_t aufs_splice_read(struct file *file, loff_t *ppos,
 
 	err = -EINVAL;
 	h_file = au_hf_top(file);
+	get_file(h_file);
 	if (au_test_loopback_kthread()) {
-		file->f_mapping = h_file->f_mapping;
-		smp_mb(); /* unnecessary? */
+		au_warn_loopback(h_file->f_dentry->d_sb);
+		if (file->f_mapping != h_file->f_mapping) {
+			file->f_mapping = h_file->f_mapping;
+			smp_mb(); /* unnecessary? */
+		}
 	}
+	di_read_unlock(dentry, AuLock_IR);
+	fi_read_unlock(file);
+
 	err = vfsub_splice_to(h_file, ppos, pipe, len, flags);
 	/* todo: necessasry? */
 	/* file->f_ra = h_file->f_ra; */
+	/* update without lock, I don't think it a problem */
 	fsstack_copy_attr_atime(dentry->d_inode, h_file->f_dentry->d_inode);
-
-	di_read_unlock(dentry, AuLock_IR);
-	fi_read_unlock(file);
+	fput(h_file);
 
 out:
 	si_read_unlock(sb);
@@ -346,137 +388,64 @@ aufs_splice_write(struct pipe_inode_info *pipe, struct file *file, loff_t *ppos,
 	struct dentry *dentry;
 	struct inode *inode;
 	struct file *h_file;
+	struct super_block *sb;
 
 	dentry = file->f_dentry;
+	sb = dentry->d_sb;
 	inode = dentry->d_inode;
 	au_mtx_and_read_lock(inode);
+
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
 	if (unlikely(err))
 		goto out;
 
 	err = au_ready_to_write(file, -1, &pin);
 	di_downgrade_lock(dentry, AuLock_IR);
-	if (unlikely(err))
-		goto out_unlock;
+	if (unlikely(err)) {
+		di_read_unlock(dentry, AuLock_IR);
+		fi_write_unlock(file);
+		goto out;
+	}
 
 	h_file = au_hf_top(file);
+	get_file(h_file);
 	au_unpin(&pin);
-	err = vfsub_splice_from(pipe, h_file, ppos, len, flags);
-	au_cpup_attr_timesizes(inode);
-	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
-
-out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
 	fi_write_unlock(file);
+
+	err = vfsub_splice_from(pipe, h_file, ppos, len, flags);
+	ii_write_lock_child(inode);
+	au_cpup_attr_timesizes(inode);
+	inode->i_mode = h_file->f_dentry->d_inode->i_mode;
+	ii_write_unlock(inode);
+	fput(h_file);
+
 out:
-	si_read_unlock(inode->i_sb);
+	si_read_unlock(sb);
 	mutex_unlock(&inode->i_mutex);
 	return err;
 }
 
 /* ---------------------------------------------------------------------- */
 
-static struct file *au_safe_file(struct vm_area_struct *vma)
-{
-	struct file *file;
-
-	file = vma->vm_file;
-	if (au_fi(file) && au_test_aufs(file->f_dentry->d_sb))
-		return file;
-	return NULL;
-}
-
-static void au_reset_file(struct vm_area_struct *vma, struct file *file)
-{
-	vma->vm_file = file;
-	/* smp_mb(); */ /* flush vm_file */
-}
-
-static int aufs_fault(struct vm_area_struct *vma, struct vm_fault *vmf)
-{
-	int err;
-	static DECLARE_WAIT_QUEUE_HEAD(wq);
-	struct file *file, *h_file;
-	struct au_finfo *finfo;
-
-	/* todo: non-robr mode, user vm_file as it is? */
-	wait_event(wq, (file = au_safe_file(vma)));
-
-	/* do not revalidate, no si lock */
-	finfo = au_fi(file);
-	AuDebugOn(finfo->fi_hdir);
-	h_file = finfo->fi_htop.hf_file;
-	AuDebugOn(!h_file || !finfo->fi_hvmop);
-
-	mutex_lock(&finfo->fi_vm_mtx);
-	vma->vm_file = h_file;
-	err = finfo->fi_hvmop->fault(vma, vmf);
-	/* todo: necessary? */
-	/* file->f_ra = h_file->f_ra; */
-	au_reset_file(vma, file);
-	mutex_unlock(&finfo->fi_vm_mtx);
-#if 0 /* def CONFIG_SMP */
-	/* wake_up_nr(&wq, online_cpu - 1); */
-	wake_up_all(&wq);
-#else
-	wake_up(&wq);
-#endif
-
-	return err;
-}
-
-static int aufs_page_mkwrite(struct vm_area_struct *vma, struct vm_fault *vmf)
-{
-	int err;
-	static DECLARE_WAIT_QUEUE_HEAD(wq);
-	struct file *file, *h_file;
-	struct au_finfo *finfo;
-
-	wait_event(wq, (file = au_safe_file(vma)));
-
-	finfo = au_fi(file);
-	AuDebugOn(finfo->fi_hdir);
-	h_file = finfo->fi_htop.hf_file;
-	AuDebugOn(!h_file || !finfo->fi_hvmop);
-
-	mutex_lock(&finfo->fi_vm_mtx);
-	vma->vm_file = h_file;
-	err = finfo->fi_hvmop->page_mkwrite(vma, vmf);
-	au_reset_file(vma, file);
-	mutex_unlock(&finfo->fi_vm_mtx);
-	wake_up(&wq);
-
-	return err;
-}
-
-static void aufs_vm_close(struct vm_area_struct *vma)
-{
-	static DECLARE_WAIT_QUEUE_HEAD(wq);
-	struct file *file, *h_file;
-	struct au_finfo *finfo;
-
-	wait_event(wq, (file = au_safe_file(vma)));
-
-	finfo = au_fi(file);
-	AuDebugOn(finfo->fi_hdir);
-	h_file = finfo->fi_htop.hf_file;
-	AuDebugOn(!h_file || !finfo->fi_hvmop);
-
-	mutex_lock(&finfo->fi_vm_mtx);
-	vma->vm_file = h_file;
-	finfo->fi_hvmop->close(vma);
-	au_reset_file(vma, file);
-	mutex_unlock(&finfo->fi_vm_mtx);
-	wake_up(&wq);
-}
-
-const struct vm_operations_struct aufs_vm_ops = {
-	.close		= aufs_vm_close,
-	.fault		= aufs_fault,
-	.page_mkwrite	= aufs_page_mkwrite
-};
-
-/* ---------------------------------------------------------------------- */
+/*
+ * The locking order around current->mmap_sem.
+ * - in most and regular cases
+ *   file I/O syscall -- aufs_read() or something
+ *	-- si_rwsem for read -- mmap_sem
+ *	(Note that [fdi]i_rwsem are released before mmap_sem).
+ * - in mmap case
+ *   mmap(2) -- mmap_sem -- aufs_mmap() -- si_rwsem for read -- [fdi]i_rwsem
+ * This AB-BA order is definitly bad, but is not a problem since "si_rwsem for
+ * read" allows muliple processes to acquire it and [fdi]i_rwsem are not held in
+ * file I/O. Aufs needs to stop lockdep in aufs_mmap() though.
+ * It means that when aufs acquires si_rwsem for write, the process should never
+ * acquire mmap_sem.
+ *
+ * Actually aufs_readdir() holds [fdi]i_rwsem before mmap_sem, but this is not a
+ * problem either since any directory is not able to be mmap-ed.
+ * The similar scenario is applied to aufs_readlink() too.
+ */
 
 /* cf. linux/include/linux/mman.h: calc_vm_prot_bits() */
 #define AuConv_VM_PROT(f, b)	_calc_vm_trans(f, VM_##b, PROT_##b)
@@ -513,197 +482,80 @@ static unsigned long au_flag_conv(unsigned long flags)
 		| AuConv_VM_MAP(flags, LOCKED);
 }
 
-static struct vm_operations_struct *
-au_hvmop(struct file *h_file, struct vm_area_struct *vma, unsigned long *flags)
+static int aufs_mmap(struct file *file, struct vm_area_struct *vma)
 {
-	struct vm_operations_struct *h_vmop;
+	int err;
 	unsigned long prot;
-	int err;
-
-	h_vmop = ERR_PTR(-ENODEV);
-	if (!h_file->f_op || !h_file->f_op->mmap)
-		goto out;
-
-	prot = au_prot_conv(vma->vm_flags);
-	err = security_file_mmap(h_file, /*reqprot*/prot, prot,
-				 au_flag_conv(vma->vm_flags), vma->vm_start, 0);
-	h_vmop = ERR_PTR(err);
-	if (unlikely(err))
-		goto out;
-
-	err = h_file->f_op->mmap(h_file, vma);
-	h_vmop = ERR_PTR(err);
-	if (unlikely(err))
-		goto out;
-
-	/* oops, it became 'const' */
-	h_vmop = (struct vm_operations_struct *)vma->vm_ops;
-	*flags = vma->vm_flags;
-	err = do_munmap(current->mm, vma->vm_start,
-			vma->vm_end - vma->vm_start);
-	if (unlikely(err)) {
-		AuIOErr("failed internal unmapping %.*s, %d\n",
-			AuDLNPair(h_file->f_dentry), err);
-		h_vmop = ERR_PTR(-EIO);
-	}
-
-out:
-	return h_vmop;
-}
-
-/*
- * This is another ugly approach to keep the lock order, particularly
- * mm->mmap_sem and aufs rwsem. The previous approach was reverted and you can
- * find it in git-log, if you want.
- *
- * native readdir: i_mutex, copy_to_user, mmap_sem
- * aufs readdir: i_mutex, rwsem, nested-i_mutex, copy_to_user, mmap_sem
- *
- * Before aufs_mmap() mmap_sem is acquired already, but aufs_mmap() has to
- * acquire aufs rwsem. It introduces a circular locking dependency.
- * To address this problem, aufs_mmap() delegates the part which requires aufs
- * rwsem to its internal workqueue.
- */
-
-/* very ugly approach */
-#include "mtx.h"
-
-struct au_mmap_pre_args {
-	/* input */
-	struct file *file;
-	struct vm_area_struct *vma;
-
-	/* output */
-	int *errp;
-	struct file *h_file;
-	struct au_branch *br;
-	int mmapped;
-};
-
-static int au_mmap_pre(struct file *file, struct vm_area_struct *vma,
-		       struct file **h_file, struct au_branch **br,
-		       int *mmapped)
-{
-	int err;
 	aufs_bindex_t bstart;
 	const unsigned char wlock
-		= !!(file->f_mode & FMODE_WRITE) && (vma->vm_flags & VM_SHARED);
+		= (file->f_mode & FMODE_WRITE) && (vma->vm_flags & VM_SHARED);
 	struct dentry *dentry;
 	struct super_block *sb;
+	struct file *h_file;
+	struct au_branch *br;
+	struct au_pin pin;
+
+	AuDbgVmRegion(file, vma);
 
 	dentry = file->f_dentry;
 	sb = dentry->d_sb;
+	lockdep_off();
 	si_read_lock(sb, AuLock_NOPLMW);
 	err = au_reval_and_lock_fdi(file, au_reopen_nondir, /*wlock*/1);
 	if (unlikely(err))
 		goto out;
 
-	*mmapped = !!au_test_mmapped(file);
 	if (wlock) {
-		struct au_pin pin;
-
 		err = au_ready_to_write(file, -1, &pin);
 		di_write_unlock(dentry);
-		if (unlikely(err))
-			goto out_unlock;
+		if (unlikely(err)) {
+			fi_write_unlock(file);
+			goto out;
+		}
 		au_unpin(&pin);
 	} else
 		di_write_unlock(dentry);
-	bstart = au_fbstart(file);
-	*br = au_sbr(sb, bstart);
-	*h_file = au_hf_top(file);
-	get_file(*h_file);
-	au_fi_mmap_lock(file);
 
-out_unlock:
+	bstart = au_fbstart(file);
+	br = au_sbr(sb, bstart);
+	h_file = au_hf_top(file);
+	get_file(h_file);
+	au_set_mmapped(file);
 	fi_write_unlock(file);
+	lockdep_on();
+
+	au_vm_file_reset(vma, h_file);
+	prot = au_prot_conv(vma->vm_flags);
+	err = security_file_mmap(h_file, /*reqprot*/prot, prot,
+				 au_flag_conv(vma->vm_flags), vma->vm_start, 0);
+	if (!err)
+		err = h_file->f_op->mmap(h_file, vma);
+	if (unlikely(err))
+		goto out_reset;
+
+	au_vm_prfile_set(vma, file);
+	/* update without lock, I don't think it a problem */
+	fsstack_copy_attr_atime(file->f_dentry->d_inode,
+				h_file->f_dentry->d_inode);
+	goto out_fput; /* success */
+
+out_reset:
+	au_unset_mmapped(file);
+	au_vm_file_reset(vma, file);
+out_fput:
+	fput(h_file);
+	lockdep_off();
 out:
 	si_read_unlock(sb);
-	return err;
-}
-
-static void au_call_mmap_pre(void *args)
-{
-	struct au_mmap_pre_args *a = args;
-	*a->errp = au_mmap_pre(a->file, a->vma, &a->h_file, &a->br,
-			       &a->mmapped);
-}
-
-static int aufs_mmap(struct file *file, struct vm_area_struct *vma)
-{
-	int err, wkq_err;
-	unsigned long h_vmflags;
-	struct au_finfo *finfo;
-	struct dentry *h_dentry;
-	struct vm_operations_struct *h_vmop, *vmop;
-	struct au_mmap_pre_args args = {
-		.file		= file,
-		.vma		= vma,
-		.errp		= &err
-	};
-
-	wkq_err = au_wkq_wait_pre(au_call_mmap_pre, &args);
-	if (unlikely(wkq_err))
-		err = wkq_err;
-	if (unlikely(err))
-		goto out;
-	finfo = au_fi(file);
-	mutex_set_owner(&finfo->fi_mmap);
-
-	h_dentry = args.h_file->f_dentry;
-	if (!args.mmapped && au_test_fs_bad_mapping(h_dentry->d_sb)) {
-		/*
-		 * by this assignment, f_mapping will differs from aufs inode
-		 * i_mapping.
-		 * if someone else mixes the use of f_dentry->d_inode and
-		 * f_mapping->host, then a problem may arise.
-		 */
-		file->f_mapping = args.h_file->f_mapping;
-	}
-
-	/* always try this internal mmap to get vma flags */
-	h_vmflags = 0; /* gcc warning */
-	h_vmop = au_hvmop(args.h_file, vma, &h_vmflags);
-	err = PTR_ERR(h_vmop);
-	if (IS_ERR(h_vmop))
-		goto out_unlock;
-	AuDebugOn(args.mmapped && h_vmop != finfo->fi_hvmop);
-
-	vmop = (void *)au_dy_vmop(file, args.br, h_vmop);
-	err = PTR_ERR(vmop);
-	if (IS_ERR(vmop))
-		goto out_unlock;
-
-	/*
-	 * unnecessary to handle MAP_DENYWRITE and deny_write_access()?
-	 * currently MAP_DENYWRITE from userspace is ignored, but elf loader
-	 * sets it. when FMODE_EXEC is set (by open_exec() or sys_uselib()),
-	 * both of the aufs file and the lower file is deny_write_access()-ed.
-	 * finally I hope we can skip handlling MAP_DENYWRITE here.
-	 */
-	err = generic_file_mmap(file, vma);
-	if (unlikely(err))
-		goto out_unlock;
-
-	vma->vm_ops = vmop;
-	vma->vm_flags = h_vmflags;
-	if (!args.mmapped)
-		finfo->fi_hvmop = h_vmop;
-
-	vfsub_file_accessed(args.h_file);
-	/* update without lock, I don't think it a problem */
-	fsstack_copy_attr_atime(file->f_dentry->d_inode, h_dentry->d_inode);
-
-out_unlock:
-	au_fi_mmap_unlock(file);
-	fput(args.h_file);
-out:
+	lockdep_on();
+	AuTraceErr(err);
 	return err;
 }
 
 /* ---------------------------------------------------------------------- */
 
-static int aufs_fsync_nondir(struct file *file, int datasync)
+static int aufs_fsync_nondir(struct file *file, loff_t start, loff_t end,
+			     int datasync)
 {
 	int err;
 	struct au_pin pin;
@@ -714,14 +566,8 @@ static int aufs_fsync_nondir(struct file *file, int datasync)
 
 	dentry = file->f_dentry;
 	inode = dentry->d_inode;
-	IMustLock(file->f_mapping->host);
-	if (inode != file->f_mapping->host) {
-		mutex_unlock(&file->f_mapping->host->i_mutex);
-		mutex_lock(&inode->i_mutex);
-	}
-	IMustLock(inode);
-
 	sb = dentry->d_sb;
+	mutex_lock(&inode->i_mutex);
 	err = si_read_lock(sb, AuLock_FLUSH | AuLock_NOPLM);
 	if (unlikely(err))
 		goto out;
@@ -741,22 +587,8 @@ static int aufs_fsync_nondir(struct file *file, int datasync)
 
 	err = -EINVAL;
 	h_file = au_hf_top(file);
-	if (h_file->f_op && h_file->f_op->fsync) {
-		struct mutex *h_mtx;
-
-		/*
-		 * no filemap_fdatawrite() since aufs file has no its own
-		 * mapping, but dir.
-		 */
-		h_mtx = &h_file->f_dentry->d_inode->i_mutex;
-		mutex_lock_nested(h_mtx, AuLsc_I_CHILD);
-		err = h_file->f_op->fsync(h_file, datasync);
-		if (!err)
-			vfsub_update_h_iattr(&h_file->f_path, /*did*/NULL);
-		/*ignore*/
-		au_cpup_attr_timesizes(inode);
-		mutex_unlock(h_mtx);
-	}
+	err = vfsub_fsync(h_file, &h_file->f_path, datasync);
+	au_cpup_attr_timesizes(inode);
 
 out_unlock:
 	di_read_unlock(dentry, AuLock_IR);
@@ -764,10 +596,7 @@ out_unlock:
 out_si:
 	si_read_unlock(sb);
 out:
-	if (inode != file->f_mapping->host) {
-		mutex_unlock(&inode->i_mutex);
-		mutex_lock(&file->f_mapping->host->i_mutex);
-	}
+	mutex_unlock(&inode->i_mutex);
 	return err;
 }
 
@@ -871,12 +700,8 @@ static ssize_t aufs_sendpage(struct file *file, struct page *page, int offset,
 
 const struct file_operations aufs_file_fop = {
 	.owner		= THIS_MODULE,
-	/*
-	 * while generic_file_llseek/_unlocked() don't use BKL,
-	 * don't use it since it operates file->f_mapping->host.
-	 * in aufs, it may be a real file and may confuse users by UDBA.
-	 */
-	/* .llseek		= generic_file_llseek, */
+
+	.llseek		= default_llseek,
 
 	.read		= aufs_read,
 	.write		= aufs_write,
