@@ -1,7 +1,7 @@
 /*
  * kernel/power/tuxonice_io.h
  *
- * Copyright (C) 2005-2007 Nigel Cunningham (nigel at tuxonice net)
+ * Copyright (C) 2005-2010 Nigel Cunningham (nigel at tuxonice net)
  *
  * This file is released under the GPLv2.
  *
@@ -14,11 +14,20 @@
 
 /* Non-module data saved in our image header */
 struct toi_header {
+	/*
+	 * Mirror struct swsusp_info, but without
+	 * the page aligned attribute
+	 */
+	struct new_utsname uts;
 	u32 version_code;
 	unsigned long num_physpages;
+	int cpus;
+	unsigned long image_pages;
+	unsigned long pages;
+	unsigned long size;
+
+	/* Our own data */
 	unsigned long orig_mem_free;
-	struct new_utsname uts;
-	int num_cpus;
 	int page_size;
 	int pageset_2_size;
 	int param0;
@@ -46,6 +55,7 @@ extern void attempt_to_parse_alt_resume_param(void);
 int image_exists_read(const char *page, int count);
 int image_exists_write(const char *buffer, int count);
 extern void save_restore_alt_param(int replace, int quiet);
+extern atomic_t toi_io_workers;
 
 /* Args to save_restore_alt_param */
 #define RESTORE 0
@@ -55,3 +65,10 @@ extern void save_restore_alt_param(int replace, int quiet);
 #define QUIET 1
 
 extern dev_t name_to_dev_t(char *line);
+
+extern wait_queue_head_t toi_io_queue_flusher;
+extern int toi_bio_queue_flusher_should_finish;
+
+int fs_info_space_needed(void);
+
+extern int toi_max_workers;
