@@ -4,7 +4,7 @@
  * Copyright (C) 1998-2001 Gabor Kuti <seasons@fornax.hu>
  * Copyright (C) 1998,2001,2002 Pavel Machek <pavel@suse.cz>
  * Copyright (C) 2002-2003 Florent Chabaud <fchabaud@free.fr>
- * Copyright (C) 2006-2007 Nigel Cunningham (nigel at tuxonice net)
+ * Copyright (C) 2006-2010 Nigel Cunningham (nigel at tuxonice net)
  *
  * This file is released under the GPLv2.
  *
@@ -18,6 +18,7 @@
 #include <linux/bootmem.h>
 #include <linux/hardirq.h>
 #include <linux/sched.h>
+#include <linux/cpu.h>
 #include <asm/tlbflush.h>
 
 #include "tuxonice_pageflags.h"
@@ -25,7 +26,6 @@
 #include "tuxonice_pagedir.h"
 #include "tuxonice_prepare_image.h"
 #include "tuxonice.h"
-#include "power.h"
 #include "tuxonice_builtin.h"
 #include "tuxonice_alloc.h"
 
@@ -35,7 +35,7 @@ static struct pbe **last_low_pbe_ptr;
 
 void toi_reset_alt_image_pageset2_pfn(void)
 {
-	ptoi_pfn = max_pfn + 1;
+	memory_bm_position_reset(pageset2_map);
 }
 
 static struct page *first_conflicting_page;
@@ -44,7 +44,7 @@ static struct page *first_conflicting_page;
  * free_conflicting_pages
  */
 
-void free_conflicting_pages(void)
+static void free_conflicting_pages(void)
 {
 	while (first_conflicting_page) {
 		struct page *next =
@@ -64,29 +64,23 @@ void free_conflicting_pages(void)
 struct page *___toi_get_nonconflicting_page(int can_be_highmem)
 {
 	struct page *page;
-	int flags = TOI_ATOMIC_GFP;
+	gfp_t flags = TOI_ATOMIC_GFP;
 	if (can_be_highmem)
 		flags |= __GFP_HIGHMEM;
 
 
-	if (test_toi_state(TOI_LOADING_ALT_IMAGE) && pageset2_map.bitmap &&
-				(ptoi_pfn < (max_pfn + 2))) {
-		/*
-		 * ptoi_pfn = max_pfn + 1 when yet to find first ps2 pfn that
-		 * can be used.
-		 * 	   = 0..max_pfn when going through list.
-		 * 	   = max_pfn + 2 when gone through whole list.
-		 */
+	if (test_toi_state(TOI_LOADING_ALT_IMAGE) &&
+			pageset2_map &&
+			(ptoi_pfn != BM_END_OF_MAP)) {
 		do {
-			ptoi_pfn = get_next_bit_on(&pageset2_map, ptoi_pfn);
-			if (ptoi_pfn <= max_pfn) {
+			ptoi_pfn = memory_bm_next_pfn(pageset2_map);
+			if (ptoi_pfn != BM_END_OF_MAP) {
 				page = pfn_to_page(ptoi_pfn);
 				if (!PagePageset1(page) &&
 				    (can_be_highmem || !PageHighMem(page)))
 					return page;
-			} else
-				ptoi_pfn++;
-		} while (ptoi_pfn < max_pfn);
+			}
+		} while (ptoi_pfn != BM_END_OF_MAP);
 	}
 
 	do {
@@ -94,7 +88,7 @@ struct page *___toi_get_nonconflicting_page(int can_be_highmem)
 		if (!page) {
 			printk(KERN_INFO "Failed to get nonconflicting "
 					"page.\n");
-			return 0;
+			return NULL;
 		}
 		if (PagePageset1(page)) {
 			struct page **next = (struct page **) kmap(page);
@@ -113,7 +107,7 @@ unsigned long __toi_get_nonconflicting_page(void)
 	return page ? (unsigned long) page_address(page) : 0;
 }
 
-struct pbe *get_next_pbe(struct page **page_ptr, struct pbe *this_pbe,
+static struct pbe *get_next_pbe(struct page **page_ptr, struct pbe *this_pbe,
 		int highmem)
 {
 	if (((((unsigned long) this_pbe) & (PAGE_SIZE - 1))
@@ -131,88 +125,89 @@ struct pbe *get_next_pbe(struct page **page_ptr, struct pbe *this_pbe,
 	return this_pbe;
 }
 
-/* get_pageset1_load_addresses
+/**
+ * get_pageset1_load_addresses - generate pbes for conflicting pages
  *
- * Description: We check here that pagedir & pages it points to won't collide
- * 		with pages where we're going to restore from the loaded pages
- * 		later.
- * Returns:	Zero on success, one if couldn't find enough pages (shouldn't
- * 		happen).
- */
-
+ * We check here that pagedir & pages it points to won't collide
+ * with pages where we're going to restore from the loaded pages
+ * later.
+ *
+ * Returns:
+ *	Zero on success, one if couldn't find enough pages (shouldn't
+ *	happen).
+ **/
 int toi_get_pageset1_load_addresses(void)
 {
 	int pfn, highallocd = 0, lowallocd = 0;
 	int low_needed = pagedir1.size - get_highmem_size(pagedir1);
 	int high_needed = get_highmem_size(pagedir1);
 	int low_pages_for_highmem = 0;
-	unsigned long flags = GFP_ATOMIC | __GFP_NOWARN | __GFP_HIGHMEM;
+	gfp_t flags = GFP_ATOMIC | __GFP_NOWARN | __GFP_HIGHMEM;
 	struct page *page, *high_pbe_page = NULL, *last_high_pbe_page = NULL,
-		    *low_pbe_page;
+		    *low_pbe_page, *last_low_pbe_page = NULL;
 	struct pbe **last_high_pbe_ptr = &restore_highmem_pblist,
 		   *this_high_pbe = NULL;
-	int orig_low_pfn = max_pfn + 1, orig_high_pfn = max_pfn + 1;
+	int orig_low_pfn, orig_high_pfn;
 	int high_pbes_done = 0, low_pbes_done = 0;
-	int low_direct = 0, high_direct = 0;
-	int high_to_free, low_to_free;
+	int low_direct = 0, high_direct = 0, result = 0, i;
+	int high_page = 1, high_offset = 0, low_page = 1, low_offset = 0;
+
+	memory_bm_set_iterators(pageset1_map, 3);
+	memory_bm_position_reset(pageset1_map);
+
+	memory_bm_set_iterators(pageset1_copy_map, 2);
+	memory_bm_position_reset(pageset1_copy_map);
 
 	last_low_pbe_ptr = &restore_pblist;
 
 	/* First, allocate pages for the start of our pbe lists. */
 	if (high_needed) {
 		high_pbe_page = ___toi_get_nonconflicting_page(1);
-		if (!high_pbe_page)
-			return 1;
+		if (!high_pbe_page) {
+			result = -ENOMEM;
+			goto out;
+		}
 		this_high_pbe = (struct pbe *) kmap(high_pbe_page);
 		memset(this_high_pbe, 0, PAGE_SIZE);
 	}
 
 	low_pbe_page = ___toi_get_nonconflicting_page(0);
-	if (!low_pbe_page)
-		return 1;
+	if (!low_pbe_page) {
+		result = -ENOMEM;
+		goto out;
+	}
 	this_low_pbe = (struct pbe *) page_address(low_pbe_page);
 
 	/*
-	 * Next, allocate all possible memory to find where we can
-	 * load data directly into destination pages. I'd like to do
-	 * this in bigger chunks, but then we can't free pages
-	 * individually later.
+	 * Next, allocate the number of pages we need.
 	 */
 
+	i = low_needed + high_needed;
+
 	do {
-		page = toi_alloc_page(30, flags);
-		if (page)
-			SetPagePageset1Copy(page);
-	} while (page);
-
-	/*
-	 * Find out how many high- and lowmem pages we allocated above,
-	 * and how many pages we can reload directly to their original
-	 * location.
-	 */
-	BITMAP_FOR_EACH_SET(&pageset1_copy_map, pfn) {
 		int is_high;
-		page = pfn_to_page(pfn);
+
+		if (i == low_needed)
+			flags &= ~__GFP_HIGHMEM;
+
+		page = toi_alloc_page(30, flags);
+		BUG_ON(!page);
+
+		SetPagePageset1Copy(page);
 		is_high = PageHighMem(page);
 
 		if (PagePageset1(page)) {
-			if (test_action_state(TOI_NO_DIRECT_LOAD)) {
-				ClearPagePageset1Copy(page);
-				toi__free_page(30, page);
-				continue;
-			} else {
-				if (is_high)
-					high_direct++;
-				else
-					low_direct++;
-			}
+			if (is_high)
+				high_direct++;
+			else
+				low_direct++;
 		} else {
 			if (is_high)
 				highallocd++;
 			else
 				lowallocd++;
 		}
-	}
+	} while (--i);
 
 	high_needed -= high_direct;
 	low_needed -= low_direct;
@@ -227,14 +222,13 @@ int toi_get_pageset1_load_addresses(void)
 		low_needed += low_pages_for_highmem;
 	}
 
-	high_to_free = highallocd - high_needed;
-	low_to_free = lowallocd - low_needed;
-
 	/*
-	 * Now generate our pbes (which will be used for the atomic restore,
+	 * Now generate our pbes (which will be used for the atomic restore),
 	 * and free unneeded pages.
 	 */
-	BITMAP_FOR_EACH_SET(&pageset1_copy_map, pfn) {
+	memory_bm_position_reset(pageset1_copy_map);
+	for (pfn = memory_bm_next_pfn_index(pageset1_copy_map, 1); pfn != BM_END_OF_MAP;
+			pfn = memory_bm_next_pfn_index(pageset1_copy_map, 1)) {
 		int is_high;
 		page = pfn_to_page(pfn);
 		is_high = PageHighMem(page);
@@ -242,18 +236,6 @@ int toi_get_pageset1_load_addresses(void)
 		if (PagePageset1(page))
 			continue;
 
-		/* Free the page? */
-		if ((is_high && high_to_free) ||
-		    (!is_high && low_to_free)) {
-			ClearPagePageset1Copy(page);
-			toi__free_page(30, page);
-			if (is_high)
-				high_to_free--;
-			else
-				low_to_free--;
-			continue;
-		}
-
 		/* Nope. We're going to use this page. Add a pbe. */
 		if (is_high || low_pages_for_highmem) {
 			struct page *orig_page;
@@ -261,28 +243,32 @@ int toi_get_pageset1_load_addresses(void)
 			if (!is_high)
 				low_pages_for_highmem--;
 			do {
-				orig_high_pfn = get_next_bit_on(&pageset1_map,
-						orig_high_pfn);
-				BUG_ON(orig_high_pfn > max_pfn);
+				orig_high_pfn = memory_bm_next_pfn_index(pageset1_map, 1);
+				BUG_ON(orig_high_pfn == BM_END_OF_MAP);
 				orig_page = pfn_to_page(orig_high_pfn);
 			} while (!PageHighMem(orig_page) ||
-					load_direct(orig_page));
+					PagePageset1Copy(orig_page));
 
 			this_high_pbe->orig_address = orig_page;
 			this_high_pbe->address = page;
 			this_high_pbe->next = NULL;
+			toi_message(TOI_PAGEDIR, TOI_VERBOSE, 0, "High pbe %d/%d: %p(%d)=>%p",
+					high_page, high_offset, page, orig_high_pfn, orig_page);
 			if (last_high_pbe_page != high_pbe_page) {
 				*last_high_pbe_ptr =
 					(struct pbe *) high_pbe_page;
-				if (!last_high_pbe_page)
-					last_high_pbe_page = high_pbe_page;
-			} else
-				*last_high_pbe_ptr = this_high_pbe;
-			last_high_pbe_ptr = &this_high_pbe->next;
-			if (last_high_pbe_page != high_pbe_page) {
-				kunmap(last_high_pbe_page);
+				if (last_high_pbe_page) {
+					kunmap(last_high_pbe_page);
+					high_page++;
+					high_offset = 0;
+				} else
+					high_offset++;
 				last_high_pbe_page = high_pbe_page;
+			} else {
+				*last_high_pbe_ptr = this_high_pbe;
+				high_offset++;
 			}
+			last_high_pbe_ptr = &this_high_pbe->next;
 			this_high_pbe = get_next_pbe(&high_pbe_page,
 					this_high_pbe, 1);
 			if (IS_ERR(this_high_pbe)) {
@@ -294,20 +280,30 @@ int toi_get_pageset1_load_addresses(void)
 			struct page *orig_page;
 			low_pbes_done++;
 			do {
-				orig_low_pfn = get_next_bit_on(&pageset1_map,
-						orig_low_pfn);
-				BUG_ON(orig_low_pfn > max_pfn);
+				orig_low_pfn = memory_bm_next_pfn_index(pageset1_map, 2);
+				BUG_ON(orig_low_pfn == BM_END_OF_MAP);
 				orig_page = pfn_to_page(orig_low_pfn);
 			} while (PageHighMem(orig_page) ||
-					load_direct(orig_page));
+					PagePageset1Copy(orig_page));
 
 			this_low_pbe->orig_address = page_address(orig_page);
 			this_low_pbe->address = page_address(page);
 			this_low_pbe->next = NULL;
+			toi_message(TOI_PAGEDIR, TOI_VERBOSE, 0, "Low pbe %d/%d: %p(%d)=>%p",
+					low_page, low_offset, this_low_pbe->orig_address,
+					orig_low_pfn, this_low_pbe->address);
 			*last_low_pbe_ptr = this_low_pbe;
 			last_low_pbe_ptr = &this_low_pbe->next;
 			this_low_pbe = get_next_pbe(&low_pbe_page,
 					this_low_pbe, 0);
+			if (low_pbe_page != last_low_pbe_page) {
+				if (last_low_pbe_page) {
+					low_page++;
+					low_offset = 0;
+				}
+				last_low_pbe_page = low_pbe_page;
+			} else
+				low_offset++;
 			if (IS_ERR(this_low_pbe)) {
 				printk(KERN_INFO "this_low_pbe is an error.\n");
 				return -ENOMEM;
@@ -326,7 +322,10 @@ int toi_get_pageset1_load_addresses(void)
 
 	free_conflicting_pages();
 
-	return 0;
+out:
+	memory_bm_set_iterators(pageset1_map, 1);
+	memory_bm_set_iterators(pageset1_copy_map, 1);
+	return result;
 }
 
 int add_boot_kernel_data_pbe(void)
