@@ -1,7 +1,7 @@
 /*
  * kernel/power/tuxonice_storage.h
  *
- * Copyright (C) 2005-2007 Nigel Cunningham (nigel at tuxonice net)
+ * Copyright (C) 2005-2010 Nigel Cunningham (nigel at tuxonice net)
  *
  * This file is released under the GPLv2.
  */
@@ -43,11 +43,3 @@ enum {
 
 	USM_MSG_MAX,
 };
-
-#ifdef CONFIG_NET
-extern __init int toi_usm_init(void);
-extern __exit void toi_usm_cleanup(void);
-#else
-#define toi_usm_init() do { } while(0)
-#define toi_usm_cleanup() do { } while(0)
-#endif
