@@ -4,7 +4,7 @@
  * Copyright (C) 1998-2001 Gabor Kuti <seasons@fornax.hu>
  * Copyright (C) 1998,2001,2002 Pavel Machek <pavel@suse.cz>
  * Copyright (C) 2002-2003 Florent Chabaud <fchabaud@free.fr>
- * Copyright (C) 2002-2007 Nigel Cunningham (nigel at tuxonice net)
+ * Copyright (C) 2002-2010 Nigel Cunningham (nigel at tuxonice net)
  *
  * This file is released under the GPLv2.
  *
@@ -32,10 +32,11 @@
 #include "tuxonice_ui.h"
 #include "tuxonice_netlink.h"
 #include "tuxonice_power_off.h"
+#include "tuxonice_builtin.h"
 
 static char local_printf_buf[1024];	/* Same as printk - should be safe */
-extern int toi_wait;
 struct ui_ops *toi_current_ui;
+EXPORT_SYMBOL_GPL(toi_current_ui);
 
 /**
  * toi_wait_for_keypress - Wait for keypress via userui or /dev/console.
@@ -183,6 +184,7 @@ post_ask:
 	if (continue_req)
 		set_toi_state(TOI_CONTINUE_REQ);
 }
+EXPORT_SYMBOL_GPL(toi_early_boot_message);
 #undef say
 
 /*
@@ -191,21 +193,13 @@ post_ask:
 
 static struct toi_sysfs_data sysfs_params[] = {
 #if defined(CONFIG_NET) && defined(CONFIG_SYSFS)
-	{ TOI_ATTR("default_console_level", SYSFS_RW),
-	  SYSFS_INT(&toi_bkd.toi_default_console_level, 0, 7, 0)
-	},
-
-	{ TOI_ATTR("debug_sections", SYSFS_RW),
-	  SYSFS_UL(&toi_bkd.toi_debug_state, 0, 1 << 30, 0)
-	},
-
-	{ TOI_ATTR("log_everything", SYSFS_RW),
-	  SYSFS_BIT(&toi_bkd.toi_action, TOI_LOGALL, 0)
-	},
+	SYSFS_INT("default_console_level", SYSFS_RW,
+			&toi_bkd.toi_default_console_level, 0, 7, 0, NULL),
+	SYSFS_UL("debug_sections", SYSFS_RW, &toi_bkd.toi_debug_state, 0,
+			1 << 30, 0),
+	SYSFS_BIT("log_everything", SYSFS_RW, &toi_bkd.toi_action, TOI_LOGALL,
+			0)
 #endif
-	{ TOI_ATTR("pm_prepare_console", SYSFS_RW),
-	  SYSFS_BIT(&toi_bkd.toi_action, TOI_PM_PREPARE_CONSOLE, 0)
-	}
 };
 
 static struct toi_module_ops userui_ops = {
@@ -230,6 +224,7 @@ int toi_register_ui_ops(struct ui_ops *this_ui)
 
 	return 0;
 }
+EXPORT_SYMBOL_GPL(toi_register_ui_ops);
 
 void toi_remove_ui_ops(struct ui_ops *this_ui)
 {
@@ -238,6 +233,7 @@ void toi_remove_ui_ops(struct ui_ops *this_ui)
 
 	toi_current_ui = NULL;
 }
+EXPORT_SYMBOL_GPL(toi_remove_ui_ops);
 
 /* toi_console_sysfs_init
  * Description: Boot time initialisation for user interface.
@@ -252,10 +248,3 @@ void toi_ui_exit(void)
 {
 	toi_unregister_module(&userui_ops);
 }
-
-#ifdef CONFIG_TOI_EXPORTS
-EXPORT_SYMBOL_GPL(toi_current_ui);
-EXPORT_SYMBOL_GPL(toi_early_boot_message);
-EXPORT_SYMBOL_GPL(toi_register_ui_ops);
-EXPORT_SYMBOL_GPL(toi_remove_ui_ops);
-#endif
