@@ -2,7 +2,7 @@
  * kernel/power/user_ui.c
  *
  * Copyright (C) 2005-2007 Bernard Blackham
- * Copyright (C) 2002-2007 Nigel Cunningham (nigel at tuxonice net)
+ * Copyright (C) 2002-2010 Nigel Cunningham (nigel at tuxonice net)
  *
  * This file is released under the GPLv2.
  *
@@ -28,11 +28,11 @@
 #include <linux/ctype.h>
 #include <linux/tty.h>
 #include <linux/vt_kern.h>
-#include <linux/module.h>
 #include <linux/reboot.h>
 #include <linux/kmod.h>
 #include <linux/security.h>
 #include <linux/syscalls.h>
+#include <linux/vt.h>
 
 #include "tuxonice_sysfs.h"
 #include "tuxonice_modules.h"
@@ -65,15 +65,20 @@ static DECLARE_WAIT_QUEUE_HEAD(userui_wait_for_key);
 static void ui_nl_set_state(int n)
 {
 	/* Only let them change certain settings */
-	static const int toi_action_mask =
+	static const u32 toi_action_mask =
 		(1 << TOI_REBOOT) | (1 << TOI_PAUSE) |
-		(1 << TOI_SLOW) | (1 << TOI_LOGALL) |
+		(1 << TOI_LOGALL) |
 		(1 << TOI_SINGLESTEP) |
 		(1 << TOI_PAUSE_NEAR_PAGESET_END);
+	static unsigned long new_action;
 
-	toi_bkd.toi_action = (toi_bkd.toi_action & (~toi_action_mask)) |
+	new_action = (toi_bkd.toi_action & (~toi_action_mask)) |
 		(n & toi_action_mask);
 
+	printk(KERN_DEBUG "n is %x. Action flags being changed from %lx "
+			"to %lx.", n, toi_bkd.toi_action, new_action);
+	toi_bkd.toi_action = new_action;
+
 	if (!test_action_state(TOI_PAUSE) &&
 			!test_action_state(TOI_SINGLESTEP))
 		wake_up_interruptible(&userui_wait_for_key);
@@ -85,7 +90,7 @@ static void ui_nl_set_state(int n)
  * Tell userui that atomic restore just occured, so that it can do things like
  * redrawing the screen, re-getting settings and so on.
  */
-static void userui_post_atomic_restore(void)
+static void userui_post_atomic_restore(struct toi_boot_kernel_data *bkd)
 {
 	toi_send_netlink_message(&ui_helper_data,
 			USERUI_MSG_POST_ATOMIC_RESTORE, NULL, 0);
@@ -154,7 +159,7 @@ static void set_ui_program_set(void)
 static int userui_memory_needed(void)
 {
 	/* ball park figure of 128 pages */
-	return (128 * PAGE_SIZE);
+	return 128 * PAGE_SIZE;
 }
 
 /**
@@ -171,14 +176,12 @@ static int userui_memory_needed(void)
  * maximum and progress granularity) where status needs to be updated.
  * This is to reduce unnecessary calls to update_status.
  */
-static unsigned long userui_update_status(unsigned long value,
-		unsigned long maximum, const char *fmt, ...)
+static u32 userui_update_status(u32 value, u32 maximum, const char *fmt, ...)
 {
-	static int last_step = -1;
+	static u32 last_step = 9999;
 	struct userui_msg_params msg;
+	u32 this_step, next_update;
 	int bitshift;
-	int this_step;
-	unsigned long next_update;
 
 	if (ui_helper_data.pid == -1)
 		return 0;
@@ -197,14 +200,14 @@ static unsigned long userui_update_status(unsigned long value,
 	 * of 65536 pixels or more?) */
 	bitshift = fls(maximum) - 16;
 	if (bitshift > 0) {
-		unsigned long temp_maximum = maximum >> bitshift;
-		unsigned long temp_value = value >> bitshift;
-		this_step = (int)
+		u32 temp_maximum = maximum >> bitshift;
+		u32 temp_value = value >> bitshift;
+		this_step = (u32)
 			(temp_value * progress_granularity / temp_maximum);
 		next_update = (((this_step + 1) * temp_maximum /
 					progress_granularity) + 1) << bitshift;
 	} else {
-		this_step = (int) (value * progress_granularity / maximum);
+		this_step = (u32) (value * progress_granularity / maximum);
 		next_update = ((this_step + 1) * maximum /
 				progress_granularity) + 1;
 	}
@@ -246,8 +249,8 @@ static unsigned long userui_update_status(unsigned long value,
  *
  * It may be called from an interrupt context - can't sleep!
  */
-static void userui_message(unsigned long section, unsigned long level,
-		int normally_logged, const char *fmt, ...)
+static void userui_message(u32 section, u32 level, u32 normally_logged,
+		const char *fmt, ...)
 {
 	struct userui_msg_params msg;
 
@@ -372,7 +375,7 @@ static void userui_abort_hibernate(int result_code, const char *fmt, ...)
 		printed_len = sprintf(local_printf_buf + printed_len,
 					" (Press SPACE to continue)");
 
-	toi_prepare_status(CLEAR_BAR, local_printf_buf);
+	toi_prepare_status(CLEAR_BAR, "%s", local_printf_buf);
 
 	if (ui_helper_data.pid != -1)
 		userui_wait_for_keypress(0);
@@ -386,7 +389,8 @@ static void userui_abort_hibernate(int result_code, const char *fmt, ...)
  */
 static void request_abort_hibernate(void)
 {
-	if (test_result_state(TOI_ABORT_REQUESTED))
+	if (test_result_state(TOI_ABORT_REQUESTED) ||
+	   !test_action_state(TOI_CAN_CANCEL))
 		return;
 
 	if (test_toi_state(TOI_NOW_RESUMING)) {
@@ -475,9 +479,9 @@ static int userui_user_rcv_msg(struct sk_buff *skb, struct nlmsghdr *nlh)
 				sizeof(toi_poweroff_method));
 		return 0;
 	case USERUI_MSG_SET_POWERDOWN_METHOD:
-		if (nlh->nlmsg_len < NLMSG_LENGTH(sizeof(int)))
+		if (nlh->nlmsg_len != NLMSG_LENGTH(sizeof(char)))
 			return -EINVAL;
-		toi_poweroff_method = (*data);
+		toi_poweroff_method = (unsigned long)(*data);
 		return 0;
 	case USERUI_MSG_GET_LOGLEVEL:
 		toi_send_netlink_message(&ui_helper_data,
@@ -491,7 +495,7 @@ static int userui_user_rcv_msg(struct sk_buff *skb, struct nlmsghdr *nlh)
 		toi_bkd.toi_default_console_level = (*data);
 		return 0;
 	case USERUI_MSG_PRINTK:
-		printk((char *) data);
+		printk(KERN_INFO "%s", (char *) data);
 		return 0;
 	}
 
@@ -539,13 +543,12 @@ static void userui_cond_pause(int pause, char *message)
  */
 static void userui_prepare_console(void)
 {
-	orig_kmsg = kmsg_redirect;
-	kmsg_redirect = fg_console + 1;
+	orig_kmsg = vt_kmsg_redirect(fg_console + 1);
 
 	ui_helper_data.pid = -1;
 
 	if (!userui_ops.enabled) {
-		printk("TuxOnIce: Userui disabled.\n");
+		printk(KERN_INFO "TuxOnIce: Userui disabled.\n");
 		return;
 	}
 
@@ -566,7 +569,7 @@ static void userui_cleanup_console(void)
 	if (ui_helper_data.pid > -1)
 		toi_netlink_close(&ui_helper_data);
 
-	kmsg_redirect = orig_kmsg;
+	vt_kmsg_redirect(orig_kmsg);
 }
 
 /*
@@ -575,26 +578,16 @@ static void userui_cleanup_console(void)
 
 static struct toi_sysfs_data sysfs_params[] = {
 #if defined(CONFIG_NET) && defined(CONFIG_SYSFS)
-	{ TOI_ATTR("enable_escape", SYSFS_RW),
-	  SYSFS_BIT(&toi_bkd.toi_action, TOI_CAN_CANCEL, 0)
-	},
-
-	{ TOI_ATTR("pause_between_steps", SYSFS_RW),
-	  SYSFS_BIT(&toi_bkd.toi_action, TOI_PAUSE, 0)
-	},
-
-	{ TOI_ATTR("enabled", SYSFS_RW),
-	  SYSFS_INT(&userui_ops.enabled, 0, 1, 0)
-	},
-
-	{ TOI_ATTR("progress_granularity", SYSFS_RW),
-	  SYSFS_INT(&progress_granularity, 1, 2048, 0)
-	},
-
-	{ TOI_ATTR("program", SYSFS_RW),
-	  SYSFS_STRING(ui_helper_data.program, 255, 0),
-	  .write_side_effect = set_ui_program_set,
-	},
+	SYSFS_BIT("enable_escape", SYSFS_RW, &toi_bkd.toi_action,
+			TOI_CAN_CANCEL, 0),
+	SYSFS_BIT("pause_between_steps", SYSFS_RW, &toi_bkd.toi_action,
+			TOI_PAUSE, 0),
+	SYSFS_INT("enabled", SYSFS_RW, &userui_ops.enabled, 0, 1, 0, NULL),
+	SYSFS_INT("progress_granularity", SYSFS_RW, &progress_granularity, 1,
+			2048, 0, NULL),
+	SYSFS_STRING("program", SYSFS_RW, ui_helper_data.program, 255, 0,
+			set_ui_program_set),
+	SYSFS_INT("debug", SYSFS_RW, &ui_helper_data.debug, 0, 1, 0, NULL)
 #endif
 };
 
@@ -607,13 +600,13 @@ static struct toi_module_ops userui_ops = {
 	.save_config_info		= userui_save_config_info,
 	.load_config_info		= userui_load_config_info,
 	.memory_needed			= userui_memory_needed,
+	.post_atomic_restore		= userui_post_atomic_restore,
 	.sysfs_data			= sysfs_params,
 	.num_sysfs_entries		= sizeof(sysfs_params) /
 		sizeof(struct toi_sysfs_data),
 };
 
 static struct ui_ops my_ui_ops = {
-	.post_atomic_restore		= userui_post_atomic_restore,
 	.update_status			= userui_update_status,
 	.message			= userui_message,
 	.prepare_status			= userui_prepare_status,
@@ -641,7 +634,7 @@ static __init int toi_user_ui_init(void)
 	ui_helper_data.netlink_id = NETLINK_TOI_USERUI;
 	ui_helper_data.name = "userspace ui";
 	ui_helper_data.rcv_msg = userui_user_rcv_msg;
-	ui_helper_data.interface_version = 7;
+	ui_helper_data.interface_version = 8;
 	ui_helper_data.must_init = 0;
 	ui_helper_data.not_ready = userui_cleanup_console;
 	init_completion(&ui_helper_data.wait_for_process);
@@ -660,6 +653,7 @@ static __init int toi_user_ui_init(void)
  */
 static __exit void toi_user_ui_exit(void)
 {
+	toi_netlink_close_complete(&ui_helper_data);
 	toi_remove_ui_ops(&my_ui_ops);
 	toi_unregister_module(&userui_ops);
 }
